@@ -18,25 +18,31 @@ package serverlessservice
 
 import (
 	"context"
+	"errors"
 	"fmt"
 	"testing"
 	"time"
 
 	// Inject the fakes for informers this reconciler depends on.
+	kubeclient "knative.dev/pkg/client/injection/kube/client"
 	_ "knative.dev/pkg/client/injection/kube/informers/core/v1/endpoints/fake"
 	_ "knative.dev/pkg/client/injection/kube/informers/core/v1/service/fake"
+	"knative.dev/pkg/logging"
+	servingclient "knative.dev/serving/pkg/client/injection/client/fake"
+	"knative.dev/serving/pkg/client/injection/ducks/autoscaling/v1alpha1/podscalable"
+	_ "knative.dev/serving/pkg/client/injection/ducks/autoscaling/v1alpha1/podscalable/fake"
 	_ "knative.dev/serving/pkg/client/injection/informers/networking/v1alpha1/serverlessservice/fake"
 
+	apierrs "k8s.io/apimachinery/pkg/api/errors"
 	"knative.dev/pkg/configmap"
 	"knative.dev/pkg/controller"
-	logtesting "knative.dev/pkg/logging/testing"
 	"knative.dev/pkg/ptr"
 	"knative.dev/pkg/system"
 	"knative.dev/serving/pkg/apis/networking"
 	nv1a1 "knative.dev/serving/pkg/apis/networking/v1alpha1"
-	rpkg "knative.dev/serving/pkg/reconciler"
+	v1 "knative.dev/serving/pkg/apis/serving/v1"
+	sksreconciler "knative.dev/serving/pkg/client/injection/reconciler/networking/v1alpha1/serverlessservice"
 	"knative.dev/serving/pkg/reconciler/serverlessservice/resources"
-	presources "knative.dev/serving/pkg/resources"
 
 	appsv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
@@ -47,12 +53,11 @@ import (
 	clientgotesting "k8s.io/client-go/testing"
 
 	. "knative.dev/pkg/reconciler/testing"
-	. "knative.dev/serving/pkg/reconciler/testing/v1alpha1"
+	. "knative.dev/serving/pkg/reconciler/testing/v1"
 	. "knative.dev/serving/pkg/testing"
 )
 
 func TestNewController(t *testing.T) {
-	defer logtesting.ClearAll()
 	ctx, _ := SetupFakeContext(t)
 	c := NewController(ctx, configmap.NewStaticWatcher())
 	if c == nil {
@@ -61,68 +66,75 @@ func TestNewController(t *testing.T) {
 }
 
 func TestReconcile(t *testing.T) {
+	retryAttempted := false
 	table := TableTest{{
-		Name:                    "bad workqueue key, Part I",
-		Key:                     "too/many/parts",
-		SkipNamespaceValidation: true,
+		Name: "bad workqueue key, Part I",
+		Key:  "too/many/parts",
 	}, {
-		Name:                    "bad workqueue key, Part II",
-		Key:                     "too-few-parts",
-		SkipNamespaceValidation: true,
+		Name: "bad workqueue key, Part II",
+		Key:  "too-few-parts",
 	}, {
-		Name:                    "key not found",
-		Key:                     "foo/not-found",
-		SkipNamespaceValidation: true,
+		Name: "key not found",
+		Key:  "foo/not-found",
 	}, {
 		Name: "steady state",
 		Key:  "steady/state",
 		Objects: []runtime.Object{
-			SKS("steady", "state", markHappy, WithPubService, WithPrivateService("state-fsdf"),
-				WithDeployRef("bar")),
+			SKS("steady", "state", markHappy, WithPubService, WithPrivateService, WithDeployRef("bar")),
 			deploy("steady", "bar"),
 			svcpub("steady", "state"),
-			svcpriv("steady", "state", svcWithName("state-fsdf")),
+			svcpriv("steady", "state"),
 			endpointspub("steady", "state", WithSubsets, withFilteredPorts(networking.BackendHTTPPort)),
-			endpointspriv("steady", "state", WithSubsets, epsWithName("state-fsdf")),
+			endpointspriv("steady", "state", WithSubsets),
 			activatorEndpoints(WithSubsets),
 		},
 	}, {
 		// This is the case for once we are scaled to zero.
-		Name: "steady switch to proxy mode",
+		// It also exercises the retry logic.
+		Name: "steady switch to proxy mode, with retry",
 		Key:  "steady/to-proxy",
 		Objects: []runtime.Object{
-			SKS("steady", "to-proxy", markHappy, WithPubService, WithPrivateService("to-proxy-deadbeef"),
+			SKS("steady", "to-proxy", markHappy, WithPubService, WithPrivateService,
 				WithDeployRef("bar"), withProxyMode),
 			deploy("steady", "bar"),
 			svcpub("steady", "to-proxy"),
-			svcpriv("steady", "to-proxy", svcWithName("to-proxy-deadbeef")),
+			svcpriv("steady", "to-proxy"),
 			endpointspub("steady", "to-proxy", withOtherSubsets, withFilteredPorts(networking.BackendHTTPPort)),
-			endpointspriv("steady", "to-proxy", epsWithName("to-proxy-deadbeef")),
+			endpointspriv("steady", "to-proxy"),
 			activatorEndpoints(WithSubsets),
 		},
+		WithReactors: []clientgotesting.ReactionFunc{
+			func(action clientgotesting.Action) (handled bool, ret runtime.Object, err error) {
+				if retryAttempted || !action.Matches("update", "serverlessservices") || action.GetSubresource() != "status" {
+					return false, nil, nil
+				}
+				retryAttempted = true
+				return true, nil, apierrs.NewConflict(v1.Resource("foo"), "bar", errors.New("foo"))
+			},
+		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
 			Object: SKS("steady", "to-proxy", WithDeployRef("bar"), markNoEndpoints,
-				withProxyMode, WithPubService, WithPrivateService("to-proxy-deadbeef")),
+				withProxyMode, WithPubService, WithPrivateService),
+		}, {
+			Object: SKS("steady", "to-proxy", WithDeployRef("bar"), markNoEndpoints,
+				withProxyMode, WithPubService, WithPrivateService),
 		}},
 		WantUpdates: []clientgotesting.UpdateActionImpl{{
 			Object: endpointspub("steady", "to-proxy", WithSubsets, withFilteredPorts(networking.BackendHTTPPort)),
 		}},
-		WantEvents: []string{
-			Eventf(corev1.EventTypeNormal, "Updated", `Successfully updated ServerlessService "steady/to-proxy"`),
-		},
 	}, {
 		// This is the case for once we are proxying for unsufficient burst capacity.
 		// It should be a no-op.
 		Name: "steady switch to proxy mode with endpoints",
 		Key:  "steady/to-proxy",
 		Objects: []runtime.Object{
-			SKS("steady", "to-proxy", markHappy, WithPubService, WithPrivateService("to-proxy-deadbeef"),
+			SKS("steady", "to-proxy", markHappy, WithPubService, WithPrivateService,
 				WithDeployRef("bar"), withProxyMode),
 			deploy("steady", "bar"),
 			svcpub("steady", "to-proxy"),
-			svcpriv("steady", "to-proxy", svcWithName("to-proxy-deadbeef")),
+			svcpriv("steady", "to-proxy"),
 			endpointspub("steady", "to-proxy", withOtherSubsets, withFilteredPorts(networking.BackendHTTPPort)),
-			endpointspriv("steady", "to-proxy", epsWithName("to-proxy-deadbeef"), WithSubsets),
+			endpointspriv("steady", "to-proxy", WithSubsets),
 			activatorEndpoints(WithSubsets),
 		},
 		WantUpdates: []clientgotesting.UpdateActionImpl{{
@@ -132,16 +144,16 @@ func TestReconcile(t *testing.T) {
 		Name: "many-private-services",
 		Key:  "many/privates",
 		Objects: []runtime.Object{
-			SKS("many", "privates", markHappy, WithPubService, WithPrivateService("privates-elegance-required"),
+			SKS("many", "privates", markHappy, WithPubService, WithPrivateService,
 				WithDeployRef("bar")),
 			deploy("many", "bar"),
 			svcpub("many", "privates"),
-			svcpriv("many", "privates", svcWithName("privates-elegance-required")),
+			svcpriv("many", "privates"),
 			svcpriv("many", "privates", svcWithName("privates-brutality-is-here")),
 			svcpriv("many", "privates", svcWithName("privates-uncharacteristically-pretty"),
 				WithK8sSvcOwnersRemoved), // unowned, should remain.
 			endpointspub("many", "privates", WithSubsets, withFilteredPorts(networking.BackendHTTPPort)),
-			endpointspriv("many", "privates", WithSubsets, epsWithName("privates-elegance-required")),
+			endpointspriv("many", "privates", WithSubsets),
 			activatorEndpoints(WithSubsets),
 		},
 		WantDeletes: []clientgotesting.DeleteActionImpl{{
@@ -161,12 +173,12 @@ func TestReconcile(t *testing.T) {
 		Key:  "public/svc-change",
 		Objects: []runtime.Object{
 			SKS("public", "svc-change", WithPubService, WithSKSReady,
-				WithPrivateService("svc-change-feedbeef"), WithDeployRef("bar")),
+				WithPrivateService, WithDeployRef("bar")),
 			deploy("public", "bar"),
 			svcpub("public", "svc-change", withTimeSelector),
-			svcpriv("public", "svc-change", svcWithName("svc-change-feedbeef")),
+			svcpriv("public", "svc-change"),
 			endpointspub("public", "svc-change", WithSubsets, withFilteredPorts(networking.BackendHTTPPort)),
-			endpointspriv("public", "svc-change", WithSubsets, epsWithName("svc-change-feedbeef")),
+			endpointspriv("public", "svc-change", WithSubsets),
 			activatorEndpoints(WithSubsets),
 		},
 		WantUpdates: []clientgotesting.UpdateActionImpl{{
@@ -177,16 +189,16 @@ func TestReconcile(t *testing.T) {
 		Key:  "private/svc-change",
 		Objects: []runtime.Object{
 			SKS("private", "svc-change", markHappy, WithPubService,
-				WithPrivateService("svc-change-fade"), WithDeployRef("baz")),
+				WithPrivateService, WithDeployRef("baz")),
 			deploy("private", "baz"),
 			svcpub("private", "svc-change"),
-			svcpriv("private", "svc-change", withTimeSelector, svcWithName("svc-change-fade")),
+			svcpriv("private", "svc-change", withTimeSelector),
 			endpointspub("private", "svc-change", withOtherSubsets, withFilteredPorts(networking.BackendHTTPPort)),
-			endpointspriv("private", "svc-change", WithSubsets, epsWithName("svc-change-fade")),
+			endpointspriv("private", "svc-change", WithSubsets),
 			activatorEndpoints(WithSubsets),
 		},
 		WantUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: svcpriv("private", "svc-change", svcWithName("svc-change-fade")),
+			Object: svcpriv("private", "svc-change"),
 		}, {
 			Object: endpointspub("private", "svc-change", WithSubsets, withFilteredPorts(networking.BackendHTTPPort)),
 		}},
@@ -200,7 +212,7 @@ func TestReconcile(t *testing.T) {
 		},
 		WantErr: true,
 		WantEvents: []string{
-			Eventf(corev1.EventTypeWarning, "UpdateFailed", `InternalError: error retrieving deployment selector spec: error fetching Pod Scalable on/blah: deployments.apps "blah" not found`),
+			Eventf(corev1.EventTypeWarning, "InternalError", `error retrieving deployment selector spec: error fetching Pod Scalable on/blah: deployments.apps "blah" not found`),
 		},
 	}, {
 		Name: "OnCreate-deployment-exists",
@@ -209,7 +221,7 @@ func TestReconcile(t *testing.T) {
 			SKS("on", "cde", WithDeployRef("blah")),
 			deploy("on", "blah"),
 			// This "has" to pre-exist, otherwise I can't populate it with subsets.
-			endpointspriv("on", "cde", WithSubsets, epsWithName("cde-00001")),
+			endpointspriv("on", "cde", WithSubsets),
 			activatorEndpoints(WithSubsets),
 		},
 		WantCreates: []runtime.Object{
@@ -219,22 +231,19 @@ func TestReconcile(t *testing.T) {
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
 			Object: SKS("on", "cde", WithDeployRef("blah"),
-				markHappy, WithPubService, WithPrivateService("cde-00001")),
+				markHappy, WithPubService, WithPrivateService),
 		}},
-		WantEvents: []string{
-			Eventf(corev1.EventTypeNormal, "Updated", `Successfully updated ServerlessService "on/cde"`),
-		},
 	}, {
 		Name:    "update-eps-fail",
 		Key:     "update-eps/failA",
 		WantErr: true,
 		Objects: []runtime.Object{
-			SKS("update-eps", "failA", WithPubService, WithPrivateService("failA-abba"), WithDeployRef("blah"), markNoEndpoints),
+			SKS("update-eps", "failA", WithPubService, WithPrivateService, WithDeployRef("blah"), markNoEndpoints),
 			deploy("update-eps", "blah"),
 			svcpub("update-eps", "failA"),
-			svcpriv("update-eps", "failA", svcWithName("failA-abba")),
+			svcpriv("update-eps", "failA"),
 			endpointspub("update-eps", "failA"),
-			endpointspriv("update-eps", "failA", WithSubsets, epsWithName("failA-abba")),
+			endpointspriv("update-eps", "failA", WithSubsets),
 			activatorEndpoints(WithSubsets),
 		},
 		WithReactors: []clientgotesting.ReactionFunc{
@@ -244,7 +253,8 @@ func TestReconcile(t *testing.T) {
 			Object: endpointspub("update-eps", "failA", WithSubsets, withFilteredPorts(networking.BackendHTTPPort)), // The attempted update.
 		}},
 		WantEvents: []string{
-			Eventf(corev1.EventTypeWarning, "UpdateFailed", "InternalError: inducing failure for update endpoints"),
+			Eventf(corev1.EventTypeWarning, "InternalError",
+				"failed to update public K8s Endpoints: inducing failure for update endpoints"),
 		},
 	}, {
 		Name:    "svc-fail-pub",
@@ -253,7 +263,7 @@ func TestReconcile(t *testing.T) {
 		Objects: []runtime.Object{
 			SKS("svc", "fail2", WithDeployRef("blah")),
 			deploy("svc", "blah"),
-			svcpriv("svc", "fail2", svcWithName("fail2-badbeef")),
+			svcpriv("svc", "fail2"),
 			endpointspriv("svc", "fail2"),
 			activatorEndpoints(WithSubsets),
 		},
@@ -261,15 +271,15 @@ func TestReconcile(t *testing.T) {
 			InduceFailure("create", "services"),
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: SKS("svc", "fail2", WithPrivateService("fail2-badbeef"), WithDeployRef("blah"),
-				markTransitioning("CreatingPublicService")),
+			Object: SKS("svc", "fail2", WithPrivateService,
+				WithDeployRef("blah"), markTransitioning("CreatingPublicService")),
 		}},
 		WantCreates: []runtime.Object{
 			svcpub("svc", "fail2"),
 		},
 		WantEvents: []string{
-			Eventf(corev1.EventTypeWarning, "UpdateFailed", "InternalError: inducing failure for create services"),
-			Eventf(corev1.EventTypeNormal, "Updated", `Successfully updated ServerlessService "svc/fail2"`),
+			Eventf(corev1.EventTypeWarning, "InternalError",
+				"failed to create public K8s Service: inducing failure for create services"),
 		},
 	}, {
 		Name:    "eps-create-fail-pub",
@@ -278,32 +288,31 @@ func TestReconcile(t *testing.T) {
 		Objects: []runtime.Object{
 			SKS("eps", "fail3", WithDeployRef("blah")),
 			deploy("eps", "blah"),
-			svcpriv("eps", "fail3", svcWithName("fail3-abbad")),
-			endpointspriv("eps", "fail3", WithSubsets, epsWithName("fail3-abbad")),
+			svcpriv("eps", "fail3"),
+			endpointspriv("eps", "fail3", WithSubsets),
 			activatorEndpoints(withOtherSubsets),
 		},
 		WithReactors: []clientgotesting.ReactionFunc{
 			InduceFailure("create", "endpoints"),
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: SKS("eps", "fail3", WithPubService, WithPrivateService("fail3-abbad"), WithDeployRef("blah"),
-				markTransitioning("CreatingPublicEndpoints")),
+			Object: SKS("eps", "fail3", WithPubService, WithPrivateService,
+				WithDeployRef("blah"), markTransitioning("CreatingPublicEndpoints")),
 		}},
 		WantCreates: []runtime.Object{
 			svcpub("eps", "fail3"),
 			endpointspub("eps", "fail3", WithSubsets, withFilteredPorts(networking.BackendHTTPPort)),
 		},
 		WantEvents: []string{
-			Eventf(corev1.EventTypeWarning, "UpdateFailed", "InternalError: inducing failure for create endpoints"),
-			Eventf(corev1.EventTypeNormal, "Updated", `Successfully updated ServerlessService "eps/fail3"`),
+			Eventf(corev1.EventTypeWarning, "InternalError", "failed to create public K8s Endpoints: inducing failure for create endpoints"),
 		},
 	}, {
 		Name: "OnCreate-no-eps",
 		Key:  "on/cneps",
 		Objects: []runtime.Object{
-			SKS("on", "cneps", WithDeployRef("blah"), WithPrivateService("cneps-incorrect")),
+			SKS("on", "cneps", WithDeployRef("blah"), WithPrivateService),
 			deploy("on", "blah"),
-			endpointspriv("on", "cneps", epsWithName("cneps-00001")),
+			endpointspriv("on", "cneps"),
 			activatorEndpoints(WithSubsets),
 		},
 		WantCreates: []runtime.Object{
@@ -313,18 +322,15 @@ func TestReconcile(t *testing.T) {
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
 			Object: SKS("on", "cneps", WithDeployRef("blah"),
-				markNoEndpoints, WithPubService, WithPrivateService("cneps-00001")),
+				markNoEndpoints, WithPubService, WithPrivateService),
 		}},
-		WantEvents: []string{
-			Eventf(corev1.EventTypeNormal, "Updated", `Successfully updated ServerlessService "on/cneps"`),
-		},
 	}, {
 		Name: "OnCreate-no-activator-eps-exist",
 		Key:  "on/cnaeps2",
 		Objects: []runtime.Object{
 			SKS("on", "cnaeps2", WithDeployRef("blah")),
 			deploy("on", "blah"),
-			endpointspriv("on", "cnaeps2", WithSubsets, epsWithName("cnaeps2-00001")),
+			endpointspriv("on", "cnaeps2", WithSubsets),
 			endpointspub("on", "cnaeps2", WithSubsets, withFilteredPorts(networking.BackendHTTPPort)),
 		},
 		WantErr: true,
@@ -334,12 +340,12 @@ func TestReconcile(t *testing.T) {
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
 			Object: SKS("on", "cnaeps2", WithDeployRef("blah"), WithPubService,
-				WithPrivateService("cnaeps2-00001"),
+				WithPrivateService,
 				markTransitioning("CreatingPublicService")),
 		}},
 		WantEvents: []string{
-			Eventf(corev1.EventTypeWarning, "UpdateFailed", `InternalError: endpoints "activator-service" not found`),
-			Eventf(corev1.EventTypeNormal, "Updated", `Successfully updated ServerlessService "on/cnaeps2"`),
+			Eventf(corev1.EventTypeWarning, "InternalError",
+				`failed to get activator service endpoints: endpoints "activator-service" not found`),
 		},
 	}, {
 		Name: "OnCreate-no-private-eps-exist",
@@ -357,12 +363,12 @@ func TestReconcile(t *testing.T) {
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
 			Object: SKS("on", "cnaeps3", WithDeployRef("blah"), WithPubService,
-				WithPrivateService("cnaeps3-00001"),
+				WithPrivateService,
 				markTransitioning("CreatingPublicService")),
 		}},
 		WantEvents: []string{
-			Eventf(corev1.EventTypeWarning, "UpdateFailed", `InternalError: endpoints "cnaeps3-00001" not found`),
-			Eventf(corev1.EventTypeNormal, "Updated", `Successfully updated ServerlessService "on/cnaeps3"`),
+			Eventf(corev1.EventTypeWarning, "InternalError",
+				`failed to get private K8s Service endpoints: endpoints "cnaeps3-private" not found`),
 		},
 	}, {
 		Name: "OnCreate-no-activator-eps-service",
@@ -370,7 +376,7 @@ func TestReconcile(t *testing.T) {
 		Objects: []runtime.Object{
 			SKS("on", "cnaeps", WithDeployRef("blah")),
 			deploy("on", "blah"),
-			endpointspriv("on", "cnaeps", WithSubsets, epsWithName("cnaeps-00001")),
+			endpointspriv("on", "cnaeps", WithSubsets),
 			endpointspub("on", "cnaeps", WithSubsets, withFilteredPorts(networking.BackendHTTPPort)),
 			activatorEndpoints(),
 		},
@@ -380,18 +386,15 @@ func TestReconcile(t *testing.T) {
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
 			Object: SKS("on", "cnaeps", WithDeployRef("blah"),
-				markHappy, WithPubService, WithPrivateService("cnaeps-00001")),
+				markHappy, WithPubService, WithPrivateService),
 		}},
-		WantEvents: []string{
-			Eventf(corev1.EventTypeNormal, "Updated", `Successfully updated ServerlessService "on/cnaeps"`),
-		},
 	}, {
 		Name: "OnCreate-no-activator-eps-proxy",
 		Key:  "on/cnaeps",
 		Objects: []runtime.Object{
 			SKS("on", "cnaeps", WithDeployRef("blah"), withProxyMode),
 			deploy("on", "blah"),
-			endpointspriv("on", "cnaeps", epsWithName("cnaeps-00001")), // This should be ignored.
+			endpointspriv("on", "cnaeps"), // This should be ignored.
 			activatorEndpoints(),
 		},
 		WantCreates: []runtime.Object{
@@ -401,11 +404,8 @@ func TestReconcile(t *testing.T) {
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
 			Object: SKS("on", "cnaeps", WithDeployRef("blah"), withProxyMode,
-				markNoEndpoints, WithPubService, WithPrivateService("cnaeps-00001")),
+				markNoEndpoints, WithPubService, WithPrivateService),
 		}},
-		WantEvents: []string{
-			Eventf(corev1.EventTypeNormal, "Updated", `Successfully updated ServerlessService "on/cnaeps"`),
-		},
 	}, {
 		Name:    "create-svc-fail-priv",
 		Key:     "svc/fail",
@@ -413,7 +413,7 @@ func TestReconcile(t *testing.T) {
 		Objects: []runtime.Object{
 			SKS("svc", "fail", WithDeployRef("blah")),
 			deploy("svc", "blah"),
-			endpointspriv("svc", "fail", epsWithName("blah-00001")),
+			endpointspriv("svc", "fail"),
 			activatorEndpoints(WithSubsets),
 		},
 		WithReactors: []clientgotesting.ReactionFunc{
@@ -426,21 +426,21 @@ func TestReconcile(t *testing.T) {
 			svcpriv("svc", "fail"),
 		},
 		WantEvents: []string{
-			Eventf(corev1.EventTypeWarning, "UpdateFailed", "InternalError: inducing failure for create services"),
-			Eventf(corev1.EventTypeNormal, "Updated", `Successfully updated ServerlessService "svc/fail"`),
+			Eventf(corev1.EventTypeWarning, "InternalError",
+				"failed to create private K8s Service: inducing failure for create services"),
 		},
 	}, {
 		Name:    "update-sks-fail",
 		Key:     "update-sks/fail4",
 		WantErr: true,
 		Objects: []runtime.Object{
-			SKS("update-sks", "fail4", WithPubService, WithPrivateService("fail4-42x"),
+			SKS("update-sks", "fail4", WithPubService, WithPrivateService,
 				WithDeployRef("blah")),
 			deploy("update-sks", "blah"),
 			svcpub("update-sks", "fail4"),
-			svcpriv("update-sks", "fail4", svcWithName("fail4-42x")),
+			svcpriv("update-sks", "fail4"),
 			endpointspub("update-sks", "fail4", WithSubsets, withFilteredPorts(networking.BackendHTTPPort)),
-			endpointspriv("update-sks", "fail4", WithSubsets, epsWithName("fail4-42x")),
+			endpointspriv("update-sks", "fail4", WithSubsets),
 			activatorEndpoints(WithSubsets),
 		},
 		WithReactors: []clientgotesting.ReactionFunc{
@@ -449,217 +449,211 @@ func TestReconcile(t *testing.T) {
 		// We still record update, but it fails.
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
 			Object: SKS("update-sks", "fail4",
-				WithDeployRef("blah"), markHappy, WithPubService, WithPrivateService("fail4-42x")),
+				WithDeployRef("blah"), markHappy, WithPubService, WithPrivateService),
 		}},
 		WantEvents: []string{
-			Eventf(corev1.EventTypeWarning, "UpdateFailed", "Failed to update status: inducing failure for update serverlessservices"),
+			Eventf(corev1.EventTypeWarning, "UpdateFailed", `Failed to update status for "fail4": inducing failure for update serverlessservices`),
 		},
 	}, {
 		Name:    "ronin-priv-service",
 		Key:     "ronin-priv-service/fail5",
 		WantErr: true,
 		Objects: []runtime.Object{
-			SKS("ronin-priv-service", "fail5", WithPubService, WithPrivateService("fail5-fender"),
+			SKS("ronin-priv-service", "fail5", WithPubService, WithPrivateService,
 				WithDeployRef("blah"), markHappy),
 			deploy("ronin-priv-service", "blah"),
 			svcpub("ronin-priv-service", "fail5"),
-			svcpriv("ronin-priv-service", "fail5", WithK8sSvcOwnersRemoved, svcWithName("fail5-fender")),
+			svcpriv("ronin-priv-service", "fail5", WithK8sSvcOwnersRemoved),
 			endpointspub("ronin-priv-service", "fail5", WithSubsets),
-			endpointspriv("ronin-priv-service", "fail5", WithSubsets, epsWithName("fail5-fender")),
+			endpointspriv("ronin-priv-service", "fail5", WithSubsets),
 			activatorEndpoints(WithSubsets),
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: SKS("ronin-priv-service", "fail5", WithPubService, WithPrivateService("fail5-fender"),
-				WithDeployRef("blah"), markUnowned("Service", "fail5-fender")),
+			Object: SKS("ronin-priv-service", "fail5", WithPubService, WithPrivateService,
+				WithDeployRef("blah"), markUnowned("Service", "fail5-private")),
 		}},
 		WantEvents: []string{
-			Eventf(corev1.EventTypeWarning, "UpdateFailed", `InternalError: SKS: fail5 does not own Service: fail5-fender`),
-			Eventf(corev1.EventTypeNormal, "Updated", `Successfully updated ServerlessService "ronin-priv-service/fail5"`),
+			Eventf(corev1.EventTypeWarning, "InternalError", `SKS: fail5 does not own Service: fail5-private`),
 		},
 	}, {
 		Name:    "ronin-pub-service",
 		Key:     "ronin-pub-service/fail6",
 		WantErr: true,
 		Objects: []runtime.Object{
-			SKS("ronin-pub-service", "fail6", WithPubService, WithPrivateService("fail6-gibson"),
+			SKS("ronin-pub-service", "fail6", WithPubService, WithPrivateService,
 				WithDeployRef("blah")),
 			deploy("ronin-pub-service", "blah"),
 			svcpub("ronin-pub-service", "fail6", WithK8sSvcOwnersRemoved),
-			svcpriv("ronin-pub-service", "fail6", svcWithName("fail6-gibson")),
+			svcpriv("ronin-pub-service", "fail6"),
 			endpointspub("ronin-pub-service", "fail6", WithSubsets),
-			endpointspriv("ronin-pub-service", "fail6", WithSubsets, epsWithName("fail6-gibson")),
+			endpointspriv("ronin-pub-service", "fail6", WithSubsets),
 			activatorEndpoints(WithSubsets),
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: SKS("ronin-pub-service", "fail6", WithPubService, WithPrivateService("fail6-gibson"),
+			Object: SKS("ronin-pub-service", "fail6", WithPubService, WithPrivateService,
 				WithDeployRef("blah"), markUnowned("Service", "fail6")),
 		}},
 		WantEvents: []string{
-			Eventf(corev1.EventTypeWarning, "UpdateFailed", `InternalError: SKS: fail6 does not own Service: fail6`),
-			Eventf(corev1.EventTypeNormal, "Updated", `Successfully updated ServerlessService "ronin-pub-service/fail6"`),
+			Eventf(corev1.EventTypeWarning, "InternalError", `SKS: fail6 does not own Service: fail6`),
 		},
 	}, {
 		Name:    "ronin-pub-eps",
 		Key:     "ronin-pub-eps/fail7",
 		WantErr: true,
 		Objects: []runtime.Object{
-			SKS("ronin-pub-eps", "fail7", WithPubService, WithPrivateService("fail7-schecter"),
+			SKS("ronin-pub-eps", "fail7", WithPubService, WithPrivateService,
 				WithDeployRef("blah")),
 			deploy("ronin-pub-eps", "blah"),
 			svcpub("ronin-pub-eps", "fail7"),
-			svcpriv("ronin-pub-eps", "fail7", svcWithName("fail7-schecter")),
+			svcpriv("ronin-pub-eps", "fail7"),
 			endpointspub("ronin-pub-eps", "fail7", WithSubsets, WithEndpointsOwnersRemoved),
-			endpointspriv("ronin-pub-eps", "fail7", WithSubsets, epsWithName("fail7-schecter")),
+			endpointspriv("ronin-pub-eps", "fail7", WithSubsets),
 			activatorEndpoints(WithSubsets),
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: SKS("ronin-pub-eps", "fail7", WithPubService, WithPrivateService("fail7-schecter"),
+			Object: SKS("ronin-pub-eps", "fail7", WithPubService, WithPrivateService,
 				WithDeployRef("blah"), markUnowned("Endpoints", "fail7")),
 		}},
 		WantEvents: []string{
-			Eventf(corev1.EventTypeWarning, "UpdateFailed", `InternalError: SKS: fail7 does not own Endpoints: fail7`),
-			Eventf(corev1.EventTypeNormal, "Updated", `Successfully updated ServerlessService "ronin-pub-eps/fail7"`),
+			Eventf(corev1.EventTypeWarning, "InternalError", `SKS: fail7 does not own Endpoints: fail7`),
 		},
 	}, {
 		Name:    "update-priv-svc-fail",
 		Key:     "update-svc/fail9",
 		WantErr: true,
 		Objects: []runtime.Object{
-			SKS("update-svc", "fail9", WithPubService, WithPrivateService("fail9-yamaha"),
+			SKS("update-svc", "fail9", WithPubService, WithPrivateService,
 				WithDeployRef("blah")),
 			deploy("update-svc", "blah"),
 			svcpub("update-svc", "fail9"),
-			svcpriv("update-svc", "fail9", withTimeSelector, svcWithName("fail9-yamaha")),
+			svcpriv("update-svc", "fail9", withTimeSelector),
 			endpointspub("update-svc", "fail9"),
-			endpointspriv("update-svc", "fail9", epsWithName("fail9-yamaha")),
+			endpointspriv("update-svc", "fail9"),
 			activatorEndpoints(WithSubsets),
 		},
 		WithReactors: []clientgotesting.ReactionFunc{
 			InduceFailure("update", "services"),
 		},
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: SKS("update-svc", "fail9", WithPubService, WithPrivateService("fail9-yamaha"),
+			Object: SKS("update-svc", "fail9", WithPubService, WithPrivateService,
 				WithDeployRef("blah"), markTransitioning("UpdatingPrivateService")),
 		}},
 		WantUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: svcpriv("update-svc", "fail9", svcWithName("fail9-yamaha")),
+			Object: svcpriv("update-svc", "fail9"),
 		}},
 		WantEvents: []string{
-			Eventf(corev1.EventTypeWarning, "UpdateFailed", "InternalError: inducing failure for update services"),
-			Eventf(corev1.EventTypeNormal, "Updated", `Successfully updated ServerlessService "update-svc/fail9"`),
+			Eventf(corev1.EventTypeWarning, "InternalError",
+				"failed to update private K8s Service: inducing failure for update services"),
 		},
-	},
-		{
-			Name:    "update-pub-svc-fail",
-			Key:     "update-svc/fail8",
-			WantErr: true,
-			Objects: []runtime.Object{
-				SKS("update-svc", "fail8", WithPubService, WithDeployRef("blah"), markHappy, WithPrivateService("fail8-ibanez")),
-				deploy("update-svc", "blah"),
-				svcpub("update-svc", "fail8", withTimeSelector),
-				svcpriv("update-svc", "fail8", svcWithName("fail8-ibanez")),
-				endpointspub("update-svc", "fail8", WithSubsets),
-				endpointspriv("update-svc", "fail8", WithSubsets, epsWithName("fail8-ibanez")),
-				activatorEndpoints(WithSubsets),
-			},
-			WithReactors: []clientgotesting.ReactionFunc{
-				InduceFailure("update", "services"),
-			},
-			WantUpdates: []clientgotesting.UpdateActionImpl{{
-				Object: svcpub("update-svc", "fail8"),
-			}},
-			WantEvents: []string{
-				Eventf(corev1.EventTypeWarning, "UpdateFailed", "InternalError: inducing failure for update services"),
-			},
-		}, {
-			Name: "pod change",
-			Key:  "pod/change",
-			Objects: []runtime.Object{
-				SKS("pod", "change", markHappy, WithPubService, WithPrivateService("change-prs"),
-					WithDeployRef("blah")),
-				deploy("pod", "blah"),
-				svcpub("pod", "change"),
-				svcpriv("pod", "change", svcWithName("change-prs")),
-				endpointspub("pod", "change", WithSubsets),
-				endpointspriv("pod", "change", withOtherSubsets, epsWithName("change-prs")),
-				activatorEndpoints(WithSubsets),
-			},
-			WantUpdates: []clientgotesting.UpdateActionImpl{{
-				Object: endpointspub("pod", "change", withOtherSubsets, withFilteredPorts(networking.BackendHTTPPort)),
-			}},
-		}, {
-			Name: "proxy mode; pod change - activator",
-			Key:  "pod/change",
-			Objects: []runtime.Object{
-				SKS("pod", "change", markNoEndpoints, WithPubService, withHTTP2Protocol,
-					WithPrivateService("change-taylor"), withProxyMode, WithDeployRef("blah")),
-				deploy("pod", "blah"),
-				svcpub("pod", "change", withHTTP2),
-				svcpriv("pod", "change", withHTTP2Priv, svcWithName("change-taylor")),
-				endpointspub("pod", "change", WithSubsets),
-				endpointspriv("pod", "change", epsWithName("change-taylor")),
-				activatorEndpoints(withOtherSubsets),
-			},
-			WantUpdates: []clientgotesting.UpdateActionImpl{{
-				Object: endpointspub("pod", "change", withOtherSubsets, withFilteredPorts(networking.BackendHTTP2Port)),
-			}},
-		}, {
-			Name: "serving mode; serving pod comes online",
-			Key:  "pod/change",
-			Objects: []runtime.Object{
-				SKS("pod", "change", markNoEndpoints, WithPubService,
-					WithPrivateService("change-gretsch"), WithDeployRef("blah")),
-				deploy("pod", "blah"),
-				svcpub("pod", "change"),
-				svcpriv("pod", "change", svcWithName("change-gretsch")),
-				endpointspub("pod", "change", withOtherSubsets),
-				endpointspriv("pod", "change", WithSubsets, epsWithName("change-gretsch")),
-				activatorEndpoints(withOtherSubsets),
-			},
-			WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
-				Object: SKS("pod", "change",
-					WithDeployRef("blah"), markHappy, WithPubService, WithPrivateService("change-gretsch")),
-			}},
-			WantEvents: []string{
-				Eventf(corev1.EventTypeNormal, "Updated", `Successfully updated ServerlessService "pod/change"`),
-			},
-			WantUpdates: []clientgotesting.UpdateActionImpl{{
-				Object: endpointspub("pod", "change", WithSubsets, withFilteredPorts(networking.BackendHTTPPort)),
-			}},
-		}, {
-			Name: "serving mode; no backend endpoints",
-			Key:  "pod/change",
-			Objects: []runtime.Object{
-				SKS("pod", "change", WithSKSReady, WithPubService, withHTTP2Protocol,
-					WithPrivateService("change-rickenbecker"), WithDeployRef("blah")),
-				deploy("pod", "blah"),
-				svcpub("pod", "change", withHTTP2),
-				svcpriv("pod", "change", withHTTP2Priv, svcWithName("change-rickenbecker")),
-				endpointspub("pod", "change", WithSubsets),                         // We had endpoints...
-				endpointspriv("pod", "change", epsWithName("change-rickenbecker")), // but now we don't.
-				activatorEndpoints(withOtherSubsets),
-			},
-			WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
-				Object: SKS("pod", "change", withHTTP2Protocol,
-					WithDeployRef("blah"), markNoEndpoints, WithPubService, WithPrivateService("change-rickenbecker")),
-			}},
-			WantEvents: []string{
-				Eventf(corev1.EventTypeNormal, "Updated", `Successfully updated ServerlessService "pod/change"`),
-			},
-			WantUpdates: []clientgotesting.UpdateActionImpl{{
-				Object: endpointspub("pod", "change", withOtherSubsets, withFilteredPorts(networking.BackendHTTP2Port)),
-			}},
-		}}
+	}, {
+		Name:    "update-pub-svc-fail",
+		Key:     "update-svc/fail8",
+		WantErr: true,
+		Objects: []runtime.Object{
+			SKS("update-svc", "fail8", WithPubService, WithDeployRef("blah"), markHappy, WithPrivateService),
+			deploy("update-svc", "blah"),
+			svcpub("update-svc", "fail8", withTimeSelector),
+			svcpriv("update-svc", "fail8"),
+			endpointspub("update-svc", "fail8", WithSubsets),
+			endpointspriv("update-svc", "fail8", WithSubsets),
+			activatorEndpoints(WithSubsets),
+		},
+		WithReactors: []clientgotesting.ReactionFunc{
+			InduceFailure("update", "services"),
+		},
+		WantUpdates: []clientgotesting.UpdateActionImpl{{
+			Object: svcpub("update-svc", "fail8"),
+		}},
+		WantEvents: []string{
+			Eventf(corev1.EventTypeWarning, "InternalError", "failed to update public K8s Service: inducing failure for update services"),
+		},
+	}, {
+		Name: "pod change",
+		Key:  "pod/change",
+		Objects: []runtime.Object{
+			SKS("pod", "change", markHappy, WithPubService, WithPrivateService,
+				WithDeployRef("blah")),
+			deploy("pod", "blah"),
+			svcpub("pod", "change"),
+			svcpriv("pod", "change"),
+			endpointspub("pod", "change", WithSubsets),
+			endpointspriv("pod", "change", withOtherSubsets),
+			activatorEndpoints(WithSubsets),
+		},
+		WantUpdates: []clientgotesting.UpdateActionImpl{{
+			Object: endpointspub("pod", "change", withOtherSubsets, withFilteredPorts(networking.BackendHTTPPort)),
+		}},
+	}, {
+		Name: "proxy mode; pod change - activator",
+		Key:  "pod/change",
+		Objects: []runtime.Object{
+			SKS("pod", "change", markNoEndpoints, WithPubService, withHTTP2Protocol,
+				WithPrivateService, WithDeployRef("blah")),
+			deploy("pod", "blah"),
+			svcpub("pod", "change", withHTTP2),
+			svcpriv("pod", "change", withHTTP2Priv),
+			endpointspub("pod", "change", WithSubsets),
+			endpointspriv("pod", "change"),
+			activatorEndpoints(withOtherSubsets),
+		},
+		WantUpdates: []clientgotesting.UpdateActionImpl{{
+			Object: endpointspub("pod", "change", withOtherSubsets, withFilteredPorts(networking.BackendHTTP2Port)),
+		}},
+	}, {
+		Name: "serving mode; serving pod comes online",
+		Key:  "pod/change",
+		Objects: []runtime.Object{
+			SKS("pod", "change", markNoEndpoints, WithPubService,
+				WithPrivateService, WithDeployRef("blah")),
+			deploy("pod", "blah"),
+			svcpub("pod", "change"),
+			svcpriv("pod", "change"),
+			endpointspub("pod", "change", withOtherSubsets),
+			endpointspriv("pod", "change", WithSubsets),
+			activatorEndpoints(withOtherSubsets),
+		},
+		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
+			Object: SKS("pod", "change",
+				WithDeployRef("blah"), markHappy, WithPubService, WithPrivateService, WithDeployRef("blah")),
+		}},
+		WantUpdates: []clientgotesting.UpdateActionImpl{{
+			Object: endpointspub("pod", "change", WithSubsets, withFilteredPorts(networking.BackendHTTPPort)),
+		}},
+	}, {
+		Name: "serving mode; no backend endpoints",
+		Key:  "pod/change",
+		Objects: []runtime.Object{
+			SKS("pod", "change", WithSKSReady, WithPubService, withHTTP2Protocol,
+				WithPrivateService, WithDeployRef("blah")),
+			deploy("pod", "blah"),
+			svcpub("pod", "change", withHTTP2),
+			svcpriv("pod", "change", withHTTP2Priv),
+			endpointspub("pod", "change", WithSubsets), // We had endpoints...
+			endpointspriv("pod", "change"),             // but now we don't.
+			activatorEndpoints(withOtherSubsets),
+		},
+		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
+			Object: SKS("pod", "change", withHTTP2Protocol,
+				WithDeployRef("blah"), markNoEndpoints, WithPubService, WithPrivateService),
+		}},
+		WantUpdates: []clientgotesting.UpdateActionImpl{{
+			Object: endpointspub("pod", "change", withOtherSubsets, withFilteredPorts(networking.BackendHTTP2Port)),
+		}},
+	}}
 
-	defer logtesting.ClearAll()
 	table.Test(t, MakeFactory(func(ctx context.Context, listers *Listers, cmw configmap.Watcher) controller.Reconciler {
-		return &reconciler{
-			Base:              rpkg.NewBase(ctx, controllerAgentName, cmw),
-			sksLister:         listers.GetServerlessServiceLister(),
+		retryAttempted = false
+		ctx = podscalable.WithDuck(ctx)
+
+		r := &reconciler{
+			kubeclient:        kubeclient.Get(ctx),
 			serviceLister:     listers.GetK8sServiceLister(),
 			endpointsLister:   listers.GetEndpointsLister(),
-			psInformerFactory: presources.NewPodScalableInformerFactory(ctx),
+			psInformerFactory: podscalable.Get(ctx),
 		}
+
+		return sksreconciler.NewReconciler(ctx, logging.FromContext(ctx), servingclient.Get(ctx),
+			listers.GetServerlessServiceLister(), controller.GetEventRecorder(ctx), r)
 	}))
 }
 
@@ -797,12 +791,6 @@ func activatorEndpoints(eo ...EndpointsOption) *corev1.Endpoints {
 	return ep
 }
 
-func epsWithName(n string) EndpointsOption {
-	return func(e *corev1.Endpoints) {
-		e.Name = n
-	}
-}
-
 func endpointspriv(namespace, name string, eo ...EndpointsOption) *corev1.Endpoints {
 	service := svcpriv(namespace, name)
 	ep := &corev1.Endpoints{
